@@ -279,7 +279,6 @@ sub can
     my $headers = ref( $self ) ? $self->headers : 'HTTP::Promise::Headers';
     my $trace = '';
     my $debug = $self->debug // 0;
-    $trace = $self->_get_stack_trace if( $debug >= 4 );
     if( $headers->can( $method ) )
     {
         # We create the function here so that it will not need to be
@@ -992,6 +991,60 @@ sub AUTOLOAD
 # avoid AUTOLOADing it
 sub DESTROY { }
 
+# NOTE: sub FREEZE is inherited
+
+sub STORABLE_freeze { CORE::return( CORE::shift->FREEZE( @_ ) ); }
+
+sub STORABLE_thaw { CORE::return( CORE::shift->THAW( @_ ) ); }
+
+# NOTE: sub THAW is inherited
+sub THAW
+{
+    my( $self, undef, @args ) = @_;
+    my $ref = ( CORE::scalar( @args ) == 1 && CORE::ref( $args[0] ) eq 'ARRAY' ) ? CORE::shift( @args ) : \@args;
+    my $class = ( CORE::defined( $ref ) && CORE::ref( $ref ) eq 'ARRAY' && CORE::scalar( @$ref ) > 1 ) ? CORE::shift( @$ref ) : ( CORE::ref( $self ) || $self );
+    my $hash = CORE::ref( $ref ) eq 'ARRAY' ? CORE::shift( @$ref ) : {};
+    my $new;
+    # Storable pattern requires to modify the object it created rather than returning a new one
+    if( CORE::ref( $self ) )
+    {
+        foreach( CORE::keys( %$hash ) )
+        {
+            $self->{ $_ } = CORE::delete( $hash->{ $_ } );
+        }
+        
+        # Need to make sure the headers object, which is an XS one is properly post processed, because Storable does not handle well XS objects, as of version 3.26
+        if( CORE::exists( $self->{headers} ) && 
+            CORE::defined( $self->{headers} ) && 
+            CORE::ref( $self->{headers} ) && 
+            $self->{headers}->isa( 'HTTP::Promise::Headers' ) )
+        {
+            $self->{headers} = $self->{headers}->STORABLE_thaw_post_processing;
+        }
+        # The headers object in HTTP::Promise::Message must be the same shared on in HTTP::Promise::Entity
+        if( CORE::exists( $self->{entity} ) &&
+            CORE::exists( $self->{entity}->{headers} ) )
+        {
+            $self->{entity}->{headers} = $self->{headers};
+        }
+        $new = $self;
+    }
+    else
+    {
+        $new = bless( $hash => $class );
+    }
+    if( CORE::exists( $new->{entity} ) && 
+        CORE::defined( $new->{entity} ) && 
+        $new->{entity} )
+    {
+        $new->{entity}->{http_message} = $new;
+    }
+    CORE::return( $new );
+}
+
+# NOTE: only here to avoid triggering HTTP::Promise::Headers::STORABLE_thaw_post_processing which we inherit when we did 'require HTTP::Promise::Headers'
+sub STORABLE_thaw_post_processing { CORE::return( $_[0] ); }
+
 1;
 # NOTE: POD
 __END__
